'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var cosmiconfig = require('cosmiconfig');
var path = require('path');
var camelcase = require('camelcase');
var loadJsonFile = require('load-json-file');
var pluginBabel = require('@rollup/plugin-babel');
var pluginBuble = require('@rollup/plugin-buble');
var pluginCommonJS = require('@rollup/plugin-commonjs');
var pluginJSON = require('@rollup/plugin-json');
var pluginNodeResolve = require('@rollup/plugin-node-resolve');
var builtinModules = require('builtin-modules');
var pluginAddShebang = require('rollup-plugin-add-shebang');
var rollupPluginEslint = require('rollup-plugin-eslint');
var pluginEquals = require('rollup-plugin-export-equals');
var pluginStripShebang = require('rollup-plugin-strip-shebang');
var rollupPluginTerser = require('rollup-plugin-terser');
var pluginTypescript = require('rollup-plugin-typescript2');
var slash = require('slash');
var fs = require('fs');
var resolveFrom = require('resolve-from');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var camelcase__default = /*#__PURE__*/_interopDefaultLegacy(camelcase);
var loadJsonFile__default = /*#__PURE__*/_interopDefaultLegacy(loadJsonFile);
var pluginBabel__default = /*#__PURE__*/_interopDefaultLegacy(pluginBabel);
var pluginBuble__default = /*#__PURE__*/_interopDefaultLegacy(pluginBuble);
var pluginCommonJS__default = /*#__PURE__*/_interopDefaultLegacy(pluginCommonJS);
var pluginJSON__default = /*#__PURE__*/_interopDefaultLegacy(pluginJSON);
var pluginNodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(pluginNodeResolve);
var builtinModules__default = /*#__PURE__*/_interopDefaultLegacy(builtinModules);
var pluginAddShebang__default = /*#__PURE__*/_interopDefaultLegacy(pluginAddShebang);
var pluginEquals__default = /*#__PURE__*/_interopDefaultLegacy(pluginEquals);
var pluginStripShebang__default = /*#__PURE__*/_interopDefaultLegacy(pluginStripShebang);
var pluginTypescript__default = /*#__PURE__*/_interopDefaultLegacy(pluginTypescript);
var slash__default = /*#__PURE__*/_interopDefaultLegacy(slash);
var resolveFrom__default = /*#__PURE__*/_interopDefaultLegacy(resolveFrom);

function error(msg, ErrorType = TypeError) {
  return new ErrorType(msg);
}
function invalidPkgField(field, type) {
  return error(`Invalid package.json "${field}" field. It has to be of type ${type}`);
}
function invalidOptionOld(option, type) {
  return error(`Invalid "${option}" option. It has to be ${type}`);
}
function invalidOption(optionName, urlHash) {
  return error(`Invalid "${optionName}" option. Please check the documentation at https://github.com/manferlo81/bundlib#${urlHash || optionName}`);
}
function inputNotFound(type) {
  return error(`Input file not found for ${type}`);
}

const PRODUCT_NAME = 'bundlib';
const OPTION_FILE_PATHS = [...['', '.json', '.yaml', '.yml', '.js'].map(ext => `.${PRODUCT_NAME}rc${ext}`), `${PRODUCT_NAME}.config.js`];
const TS_DEF_PREFIX = 'd';
const MIN_PREFIX = 'min';

function isNull(value) {
  return value == null;
}
function isObject(value) {
  return !!value && typeof value === 'object';
}
function isString(value) {
  return typeof value === 'string';
}
function isBool(value) {
  return value === true || value === false;
}
const isArray = Array.isArray;
function isDictionary(value) {
  return isObject(value) && !isArray(value);
}

async function loadOptions(cwd, optionsFromPkgJson) {
  const manager = cosmiconfig.cosmiconfig(PRODUCT_NAME, {
    stopDir: cwd,
    searchPlaces: OPTION_FILE_PATHS,
    ignoreEmptySearchPlaces: false
  });

  if (optionsFromPkgJson) {
    if (!isString(optionsFromPkgJson)) {
      return {
        config: optionsFromPkgJson,
        filepath: null
      };
    }

    return manager.load(path.resolve(cwd, optionsFromPkgJson));
  }

  return manager.search();
}

function normalizeBooleanOption(build, key, def) {
  if (!build) {
    return def;
  }

  const {
    [key]: value
  } = build;

  if (isNull(value)) {
    return def;
  }

  return !!value;
}

function flattenMultilevel(array, target = []) {
  return array.reduce((acc, item) => {
    if (isArray(item)) {
      return flattenMultilevel(item, acc);
    }

    acc.push(item);
    return acc;
  }, target);
}

function createOneOfLiteral(...values) {
  const flatten = flattenMultilevel(values);

  if (flatten.length === 1) {
    const [expected] = flatten;
    return value => value === expected;
  }

  return value => flatten.some(expected => value === expected);
}
const composeOneOf = (...checks) => value => checks.some(check => check(value));
const isStringOrNull = composeOneOf(isNull, isString);
const isDictionaryOrNull = composeOneOf(isNull, isDictionary);

const MODULE_OPTION_KEYS = ['sourcemap', 'min'];
const isModuleOptionKey = createOneOfLiteral(MODULE_OPTION_KEYS);

const CJS_OPTION_KEYS = [MODULE_OPTION_KEYS, 'esModule', 'interop'];
const isCJSOptionKey = createOneOfLiteral(CJS_OPTION_KEYS);

const isBrowserOption = createOneOfLiteral(CJS_OPTION_KEYS, 'format', 'name', 'id', 'extend', 'globals');

function normalizeDeprecatedOption(build, key, isValidValue, def) {
  if (!build) {
    return def;
  }

  const {
    [key]: value
  } = build;

  if (isNull(value) || !isValidValue(value)) {
    return def;
  }

  return value;
}

const isTypesOptionKey = createOneOfLiteral('equals');

const keys = Object.keys;
const hasOwn = Object.prototype.hasOwnProperty;
function setProp(name, value, target) {
  target[name] = value;
  return target;
}
function keysToObject(keys, value, result) {
  return keys.reduce((result, key) => setProp(key, value, result), result || {});
}

function invalidKeys(object, list) {
  const invalid = keys(object).filter(key => !list.includes(key));
  return invalid.length ? invalid : null;
}
function keysCheck(obj, ...checks) {
  return keys(obj).every(checks.length === 1 ? checks[0] : composeOneOf(...checks));
}

const API_BUILD_KEYS = ['main', 'module', 'browser'];
const MODULE_BUILD_KEYS = [...API_BUILD_KEYS, 'bin'];
const ALL_BUILD_KEYS = [...MODULE_BUILD_KEYS, 'types'];
function populateWithAPIValue(value, result) {
  return keysToObject(API_BUILD_KEYS, value, result);
}
function resolveObjectSelectiveOption(value, defaultValue, isValidValue, isBuildType, allKeys, invalid) {
  if (!keysCheck(value, createOneOfLiteral('default', 'api'), isBuildType)) {
    throw invalid;
  }

  const {
    default: override,
    api,
    ...others
  } = value;

  if (!isNull(override) && !isValidValue(override)) {
    throw invalid;
  }

  const result = keysToObject(allKeys, isNull(override) ? defaultValue : override);

  if (!isNull(api)) {
    if (!isValidValue(api)) {
      throw invalid;
    }

    populateWithAPIValue(api, result);
  }

  keys(others).forEach(type => {
    const value = others[type];

    if (!isNull(value)) {
      if (!isValidValue(value)) {
        throw invalid;
      }

      result[type] = value;
    }
  });
  return result;
}
function resolveObjectBasedSelectiveOption(value, defaultValue, isValidValue, allKeys, optionName, urlHash) {
  if (isNull(value)) {
    return keysToObject(allKeys, defaultValue);
  }

  if (isValidValue(value)) {
    return keysToObject(allKeys, value);
  }

  const invalid = invalidOption(optionName, urlHash);

  if (!isObject(value)) {
    throw invalid;
  }

  return resolveObjectSelectiveOption(value, defaultValue, isValidValue, createOneOfLiteral(allKeys), allKeys, invalid);
}

function resolveTypeStringArray(value, isBuildType, allKeys, invalid) {
  return value.reduce((result, type) => {
    if (type === 'api') {
      return populateWithAPIValue(true, result);
    }

    if (!isBuildType(type)) {
      throw invalid;
    }

    return setProp(type, true, result);
  }, keysToObject(allKeys, false));
}

function resolveSelectiveOption(value, defaultValue, isValidValue, allKeys, optionName, urlHash) {
  if (isNull(value) || value === defaultValue) {
    return keysToObject(allKeys, defaultValue);
  }

  if (isValidValue(value)) {
    return keysToObject(allKeys, value);
  }

  if (value === 'api') {
    return populateWithAPIValue(true, keysToObject(allKeys, false));
  }

  const isBuildType = createOneOfLiteral(allKeys);

  if (isBuildType(value)) {
    return setProp(value, true, keysToObject(allKeys, false));
  }

  const invalid = invalidOption(optionName, urlHash);

  if (!isObject(value)) {
    throw invalid;
  }

  if (isArray(value)) {
    return resolveTypeStringArray(value, isBuildType, allKeys, invalid);
  }

  return resolveObjectSelectiveOption(value, defaultValue, isValidValue, isBuildType, allKeys, invalid);
}

const resolveESModuleOption = value => resolveSelectiveOption(value, false, isBool, MODULE_BUILD_KEYS, 'esModule', 'esmodule');

const isBrowserFormat = composeOneOf(isNull, createOneOfLiteral('iife', 'amd', 'umd'));

function isValidGlobals(value) {
  return isNull(value) || isObject(value) && (isArray(value) ? value.every(item => isString(item)) : keys(value).every(key => isString(key) && isString(value[key])));
}
function normalizeGlobals(globals) {
  return !globals ? null : isArray(globals) ? globals.reduce((result, value) => setProp(value, value, result), {}) : globals;
}
function normalizeBuildGlobals(build, def) {
  return !build || isNull(build.globals) ? def : normalizeGlobals(build.globals);
}

const resolveInputOption = value => resolveObjectBasedSelectiveOption(value, null, isString, MODULE_BUILD_KEYS, 'input');

const resolveInteropOption = value => resolveSelectiveOption(value, false, isBool, MODULE_BUILD_KEYS, 'interop');

const resolveMinOption = value => resolveSelectiveOption(value, false, isBool, MODULE_BUILD_KEYS, 'min');

function normalizeBuildName(cwd, browserName, nameOption, pkgName) {
  return browserName || nameOption || pkgName && camelcase__default['default'](path.basename(pkgName)) || camelcase__default['default'](path.basename(cwd)) || null;
}

function resolveProjectOption(value) {
  return resolveObjectBasedSelectiveOption(value, null, isString, MODULE_BUILD_KEYS, 'project');
}

function resolveSkipOption(value) {
  return resolveSelectiveOption(value, false, isBool, ALL_BUILD_KEYS, 'skip');
}

const isSourcemapOption = composeOneOf(createOneOfLiteral('inline', 'hidden'), isBool);
const resolveSourcemapOption = value => resolveSelectiveOption(value, true, isSourcemapOption, MODULE_BUILD_KEYS, 'sourcemap');

function readPkg(cwd) {
  return loadJsonFile__default['default'](path.join(cwd, 'package.json'));
}

async function analyzePkg(cwd, inputPkg) {
  const pkg = inputPkg || (await readPkg(cwd));

  if (!isDictionary(pkg)) {
    throw error('Invalid package.json content');
  }

  const {
    name: packageName,
    main: mainOutputFile,
    module: moduleFieldValue,
    'jsnext:main': jsNextFieldValue,
    browser: browserOutputFile,
    bin: binaryOutputFile,
    types: typesFieldValue,
    typings,
    dependencies: runtimeDependencies,
    devDependencies,
    peerDependencies
  } = pkg;
  const loadedOptions = await loadOptions(cwd, pkg.bundlib);
  const loadedBundlibOptions = loadedOptions && loadedOptions.config;

  if (loadedOptions && !isNull(loadedBundlibOptions) && !isDictionary(loadedBundlibOptions)) {
    throw loadedOptions.filepath ? error(`Invalid options found on file "${loadedOptions.filepath}".`) : invalidPkgField('bundlib', 'Object | string');
  }

  const bundlibOptions = loadedBundlibOptions || {};
  const invalidOptions = invalidKeys(bundlibOptions, ['input', 'extend', 'esModule', 'interop', 'sourcemap', 'format', 'name', 'id', 'globals', 'min', 'cache', 'project', 'skip', 'equals', 'main', 'module', 'browser', 'bin', 'types']);

  if (invalidOptions) {
    const optionNames = invalidOptions.map(name => `"${name}"`).join(', ');
    throw error(`Unknown options found: (${optionNames})`);
  }

  const {
    extend,
    format: browserFormat,
    name: browserName,
    id: amdId,
    globals: browserGlobals,
    cache: cacheOption,
    main: deprecatedMainOptions,
    module: deprecatedModuleOptions,
    browser: deprecatedBrowserOptions,
    bin: deprecatedBinaryOptions,
    types: deprecatedTypesOptions,
    equals
  } = bundlibOptions;
  const perBuildInput = resolveInputOption(bundlibOptions.input);
  const perBuildSourcemap = resolveSourcemapOption(bundlibOptions.sourcemap);
  const perBuildESModule = resolveESModuleOption(bundlibOptions.esModule);
  const perBuildInterop = resolveInteropOption(bundlibOptions.interop);
  const perBuildMin = resolveMinOption(bundlibOptions.min);

  if (!isBrowserFormat(browserFormat)) {
    throw invalidOption('format');
  }

  if (!isStringOrNull(browserName)) {
    throw invalidOption('name');
  }

  if (!isStringOrNull(amdId)) {
    throw invalidOption('id');
  }

  if (!isValidGlobals(browserGlobals)) {
    throw invalidOption('globals');
  }

  if (!isStringOrNull(cacheOption)) {
    throw invalidOption('cache');
  }

  const perBuildProject = resolveProjectOption(bundlibOptions.project);
  const skipBuild = resolveSkipOption(bundlibOptions.skip);

  if (!isNull(deprecatedMainOptions) && deprecatedMainOptions !== false && !(isDictionary(deprecatedMainOptions) && keysCheck(deprecatedMainOptions, isCJSOptionKey))) {
    throw invalidOptionOld('main', 'false | { sourcemap?: boolean | "inline", esModule?: boolean, interop?: boolean, min?: boolean }');
  }

  if (!isNull(deprecatedModuleOptions) && deprecatedModuleOptions !== false && !(isDictionary(deprecatedModuleOptions) && keysCheck(deprecatedModuleOptions, isModuleOptionKey))) {
    throw invalidOptionOld('module', 'false | { sourcemap?: boolean | "inline", min?: boolean }');
  }

  if (!isNull(deprecatedBrowserOptions) && deprecatedBrowserOptions !== false && !(isDictionary(deprecatedBrowserOptions) && keysCheck(deprecatedBrowserOptions, isBrowserOption) && isBrowserFormat(deprecatedBrowserOptions.format) && ['name', 'id'].every(key => isStringOrNull(deprecatedBrowserOptions[key])) && isValidGlobals(deprecatedBrowserOptions.globals))) {
    throw invalidOptionOld('browser', 'false | { sourcemap?: boolean | "inline", esModule?: boolean, interop?: boolean, min?: boolean, ... }');
  }

  if (!isNull(deprecatedBinaryOptions) && deprecatedBinaryOptions !== false && !(isDictionary(deprecatedBinaryOptions) && keysCheck(deprecatedBinaryOptions, isCJSOptionKey))) {
    throw invalidOptionOld('bin', 'false | { sourcemap?: boolean | "inline", esModule?: boolean, interop?: boolean, min?: boolean }');
  }

  if (!isNull(deprecatedTypesOptions) && deprecatedTypesOptions !== false && !(isDictionary(deprecatedTypesOptions) && keysCheck(deprecatedTypesOptions, isTypesOptionKey))) {
    throw invalidOptionOld('types', 'false | { equals?: boolean }');
  }

  if (deprecatedMainOptions !== false && !isStringOrNull(mainOutputFile)) {
    throw invalidPkgField('main', 'string');
  }

  if (deprecatedModuleOptions !== false && !isStringOrNull(moduleFieldValue)) {
    throw invalidPkgField('module', 'string');
  }

  if (!moduleFieldValue && deprecatedModuleOptions !== false && !isStringOrNull(jsNextFieldValue)) {
    throw invalidPkgField('jsnext:main', 'string');
  }

  if (deprecatedBrowserOptions !== false && !isStringOrNull(browserOutputFile)) {
    throw invalidPkgField('browser', 'string');
  }

  if (deprecatedBinaryOptions !== false && !isStringOrNull(binaryOutputFile)) {
    throw invalidPkgField('bin', 'string');
  }

  if (!isDictionaryOrNull(runtimeDependencies)) {
    throw invalidPkgField('dependencies', 'Object');
  }

  if (!isDictionaryOrNull(peerDependencies)) {
    throw invalidPkgField('peerDependencies', 'Object');
  }

  const moduleOutputFile = moduleFieldValue || jsNextFieldValue;
  const typesOutputFile = typesFieldValue || typings;
  const mainOutput = deprecatedMainOptions === false || skipBuild.main || !mainOutputFile ? null : {
    input: perBuildInput.main,
    output: mainOutputFile,
    sourcemap: normalizeDeprecatedOption(deprecatedMainOptions, 'sourcemap', isSourcemapOption, perBuildSourcemap.main),
    esModule: normalizeBooleanOption(deprecatedMainOptions, 'esModule', perBuildESModule.main),
    interop: normalizeBooleanOption(deprecatedMainOptions, 'interop', perBuildInterop.main),
    min: normalizeBooleanOption(deprecatedMainOptions, 'min', perBuildMin.main),
    project: perBuildProject.main
  };
  const moduleOutput = deprecatedModuleOptions === false || skipBuild.module || !moduleOutputFile ? null : {
    input: perBuildInput.module,
    output: moduleOutputFile,
    sourcemap: normalizeDeprecatedOption(deprecatedModuleOptions, 'sourcemap', isSourcemapOption, perBuildSourcemap.module),
    esModule: perBuildESModule.module,
    interop: perBuildInterop.module,
    min: normalizeBooleanOption(deprecatedModuleOptions, 'min', perBuildMin.module),
    project: perBuildProject.module
  };
  const browserOutput = deprecatedBrowserOptions === false || skipBuild.browser || !browserOutputFile ? null : {
    input: perBuildInput.browser,
    output: browserOutputFile,
    sourcemap: normalizeDeprecatedOption(deprecatedBrowserOptions, 'sourcemap', isSourcemapOption, perBuildSourcemap.browser),
    esModule: normalizeBooleanOption(deprecatedBrowserOptions, 'esModule', perBuildESModule.browser),
    interop: normalizeBooleanOption(deprecatedBrowserOptions, 'interop', perBuildInterop.browser),
    min: normalizeBooleanOption(deprecatedBrowserOptions, 'min', perBuildMin.browser),
    format: (deprecatedBrowserOptions && !isNull(deprecatedBrowserOptions.format) ? deprecatedBrowserOptions.format : browserFormat) || 'umd',
    name: normalizeBuildName(cwd, deprecatedBrowserOptions ? deprecatedBrowserOptions.name : null, browserName, packageName),
    id: deprecatedBrowserOptions && deprecatedBrowserOptions.id || amdId || null,
    globals: normalizeBuildGlobals(deprecatedBrowserOptions, normalizeGlobals(browserGlobals)),
    extend: !!normalizeBooleanOption(deprecatedBrowserOptions, 'extend', !!extend),
    project: perBuildProject.browser
  };
  const binaryOutput = deprecatedBinaryOptions === false || skipBuild.bin || !binaryOutputFile ? null : {
    input: perBuildInput.bin,
    output: binaryOutputFile,
    sourcemap: normalizeDeprecatedOption(deprecatedBinaryOptions, 'sourcemap', isSourcemapOption, perBuildSourcemap.bin),
    esModule: normalizeBooleanOption(deprecatedBinaryOptions, 'esModule', perBuildESModule.bin),
    interop: normalizeBooleanOption(deprecatedBinaryOptions, 'interop', perBuildInterop.bin),
    min: normalizeBooleanOption(deprecatedBinaryOptions, 'min', perBuildMin.bin),
    project: perBuildProject.bin
  };
  const typesOutput = deprecatedTypesOptions === false || skipBuild.types || !typesOutputFile ? null : {
    output: typesOutputFile,
    equals: !!equals
  };
  const dependencies = {
    runtime: runtimeDependencies || null,
    dev: devDependencies || null,
    peer: peerDependencies || null
  };
  const cache = cacheOption || null;
  return {
    cwd,
    pkg,
    main: mainOutput,
    module: moduleOutput,
    browser: browserOutput,
    bin: binaryOutput,
    types: typesOutput,
    dependencies,
    cache
  };
}

const TS_ONLY_EXTENSIONS = ['.ts', '.tsx'];
const JS_EXTENSIONS = ['.js', '.jsx', '.mjs', '.es6', '.es', '.node'];
const TS_EXTENSIONS = [...TS_ONLY_EXTENSIONS, ...JS_EXTENSIONS];

function apiPlugin(cwd, outputDir, extensions, map) {
  const resolvedMap = keys(map).reduce((resolvedMap, source) => {
    const value = slash__default['default'](path.relative(outputDir, path.resolve(cwd, map[source])));
    return value ? setProp(path.resolve(cwd, source), value.startsWith('.') ? value : `./${value}`, resolvedMap) : resolvedMap;
  }, {});

  function findTarget(resolved) {
    if (resolvedMap[resolved]) {
      return resolvedMap[resolved];
    }

    for (let i = 0; i < extensions.length; i++) {
      const ext = extensions[i];
      const sourceWithExt = resolved + ext;

      if (resolvedMap[sourceWithExt]) {
        return resolvedMap[sourceWithExt];
      }

      const sourceWithIndex = path.join(resolved, `index${ext}`);

      if (resolvedMap[sourceWithIndex]) {
        return resolvedMap[sourceWithIndex];
      }
    }
  }

  return {
    name: 'api',

    resolveId(moduleId, from) {
      if (!from) {
        return null;
      }

      const id = findTarget(path.join(path.dirname(from), moduleId));
      return id ? {
        id,
        external: true,
        moduleSideEffects: false
      } : null;
    }

  };
}

function createConfig(input, output, external, plugins, onwarn, chokidar) {
  const watch = {
    exclude: ['node_modules/**']
  };

  if (chokidar) {
    watch.chokidar = {};
  }

  const config = {
    input,
    output,
    external,
    plugins,
    watch
  };

  if (onwarn) {
    config.onwarn = onwarn;
  }

  return config;
}

function createFindInput(cwd, extensions) {
  const inputSearch = extensions.map(ext => path.resolve(cwd, 'src', `index${ext}`));
  const cache = {
    found: false,
    filename: null
  };
  return input => {
    if (input) {
      return path.resolve(cwd, input);
    }

    if (cache.found) {
      return cache.filename;
    }

    for (const filename of inputSearch) {
      if (fs.existsSync(filename)) {
        cache.found = true;
        return cache.filename = filename;
      }
    }

    cache.found = true;
  };
}

function createImportFromCWD(cwd) {
  return (id, name) => {
    const content = require(resolveFrom__default['default'](cwd, id));

    return name ? content[name] : 'default' in content ? content.default : content;
  };
}

function createIsExternal(...dependencies) {
  const filtered = dependencies.filter(dep => !!dep);

  if (!filtered.length) {
    return () => false;
  }

  const asObj = filtered.reduce((result, dep) => isArray(dep) ? keysToObject(dep, true, result) : { ...result,
    ...dep
  }, {});
  const asList = keys(asObj);
  const cache = {};
  return (source, importer, isResolved) => {
    if (isResolved || source.startsWith('.')) {
      return false;
    }

    if (hasOwn.call(cache, source)) {
      return cache[source];
    }

    if (asObj[source]) {
      return cache[source] = true;
    }

    const l = asList.length;

    for (let i = 0; i < l; i++) {
      const moduleName = asList[i];
      const len = moduleName.length;
      const partialSource = source.substr(0, len);

      if (/^[/\\]$/.test(source[len])) {
        if (hasOwn.call(cache, partialSource)) {
          return cache[partialSource];
        }

        if (partialSource === moduleName) {
          return cache[partialSource] = cache[source] = true;
        }
      }
    }

    return cache[source] = false;
  };
}

function createIsInstalled(...dependencies) {
  const reduced = dependencies.reduce((result, deps) => Object.assign(result, deps), {});
  return id => reduced[id];
}

function extensionMatch(filename, extensions) {
  return extensions.includes(path.extname(filename).toLowerCase());
}

function renamePre(filename, pre) {
  const {
    dir,
    name,
    ext
  } = path.parse(filename);
  return path.join(dir, `${name}.${pre}${ext}`);
}

function pkgToConfigs(analyzed, options) {
  const {
    cwd,
    main: commonjsOutput,
    module: moduleOutput,
    browser: browserOutput,
    bin: binaryOutput,
    types: typesOutput,
    dependencies,
    cache
  } = analyzed;
  const {
    dev,
    watch,
    onwarn
  } = options;
  const {
    runtime: runtimeDependencies,
    dev: devDependencies,
    peer: peerDependencies
  } = dependencies;
  const importFromCWD = createImportFromCWD(cwd);
  const bundlibCachePath = path.resolve(cwd, cache || 'node_modules/.cache/bundlib');
  const typescriptCachePath = path.join(bundlibCachePath, 'rpt2');
  const isExternal = createIsExternal(runtimeDependencies, peerDependencies, builtinModules__default['default']);
  const production = !dev;
  const isInstalled = createIsInstalled(runtimeDependencies, devDependencies);
  const useESLint = isInstalled('eslint');
  const useTypescript = isInstalled('typescript');
  const useBabel = isInstalled('@babel/core');
  const useChokidar = !!isInstalled('chokidar') && !!watch;
  const extensions = useTypescript ? TS_EXTENSIONS : JS_EXTENSIONS;
  const findInput = createFindInput(cwd, extensions);
  const include = extensions.map(ext => path.resolve(cwd, `**/*${ext}`));
  const exclude = /node_modules/;
  const configs = [];

  function createPlugins(inputFile, outputFile, rollupSourcemap, mini, browser, bin, apiInput, project) {
    const sourcemap = !!rollupSourcemap;
    const inputIsTypescript = extensionMatch(inputFile, TS_ONLY_EXTENSIONS);
    const typesExpectedFilename = configs.length === 0 && !bin && typesOutput && path.resolve(cwd, extensionMatch(typesOutput.output, ['.ts']) ? typesOutput.output : path.join(typesOutput.output, 'index.d.ts'));

    if (typesExpectedFilename && !inputIsTypescript) {
      throw error('Can\'t generate types from a non typescript file.');
    }

    const typesGeneratedFilename = renamePre(path.basename(inputFile), TS_DEF_PREFIX);

    if (typesExpectedFilename && typesGeneratedFilename !== path.basename(typesExpectedFilename)) {
      throw error('Input filename and types filename have to match.');
    }

    const declarationDir = typesExpectedFilename && path.dirname(typesExpectedFilename);
    let shebang;
    const plugins = [useESLint && rollupPluginEslint.eslint({
      include,
      exclude,
      throwOnWarning: false,
      throwOnError: false
    }), bin && pluginStripShebang__default['default']({
      capture: shebangFromFile => shebang = shebangFromFile,
      sourcemap
    }), bin && apiInput && apiPlugin(cwd, path.dirname(outputFile), extensions, setProp(apiInput, cwd, {})), pluginNodeResolve__default['default']({
      preferBuiltins: !browser,
      extensions,
      rootDir: cwd
    }), browser && pluginCommonJS__default['default']({
      sourceMap: sourcemap
    }), useTypescript && inputIsTypescript && pluginTypescript__default['default']({
      cwd,
      typescript: importFromCWD('typescript'),
      cacheRoot: typescriptCachePath,
      useTsconfigDeclarationDir: true,
      tsconfigDefaults: {
        exclude: []
      },
      tsconfigOverride: {
        compilerOptions: {
          sourceMap: sourcemap,
          declaration: !!declarationDir,
          ...(declarationDir && {
            declarationDir
          })
        }
      },
      ...(project && {
        tsconfig: path.resolve(cwd, project)
      })
    }), pluginJSON__default['default']({
      preferConst: true
    }), declarationDir && typesOutput && typesOutput.equals && pluginEquals__default['default']({
      file: path.resolve(cwd, path.join(declarationDir, typesGeneratedFilename))
    }), useBabel ? pluginBabel__default['default']({
      extensions,
      exclude,
      babelHelpers: 'bundled'
    }) : pluginBuble__default['default']({
      transforms: {
        dangerousForOf: true
      }
    }), bin && pluginAddShebang__default['default']({
      include: outputFile,
      shebang: () => shebang || '#!/usr/bin/env node'
    }), mini && rollupPluginTerser.terser({
      toplevel: true,
      module: true,
      compress: {
        passes: 2
      }
    })];
    return plugins.filter(Boolean);
  }

  if (commonjsOutput) {
    const {
      input,
      output,
      sourcemap,
      esModule,
      interop,
      min,
      project
    } = commonjsOutput;
    const inputFile = findInput(input);

    if (!inputFile) {
      throw inputNotFound('CommonJS module');
    }

    const outputFile = path.resolve(cwd, output);
    const outputOptions = {
      file: outputFile,
      format: 'cjs',
      sourcemap,
      esModule,
      interop,
      exports: 'auto'
    };
    configs.push(createConfig(inputFile, outputOptions, isExternal, createPlugins(inputFile, outputFile, sourcemap, production && !min, false, false, null, project), onwarn, useChokidar));

    if (min) {
      const minOutputFile = renamePre(outputFile, MIN_PREFIX);
      configs.push(createConfig(inputFile, { ...outputOptions,
        file: minOutputFile
      }, isExternal, createPlugins(inputFile, minOutputFile, sourcemap, true, false, false, null, project), onwarn, useChokidar));
    }
  }

  if (moduleOutput) {
    const {
      input,
      output,
      sourcemap,
      esModule,
      interop,
      min,
      project
    } = moduleOutput;
    const inputFile = findInput(input);

    if (!inputFile) {
      throw inputNotFound('ES module');
    }

    const outputFile = path.resolve(cwd, output);
    const outputOptions = {
      file: outputFile,
      format: 'es',
      sourcemap,
      esModule,
      interop
    };
    configs.push(createConfig(inputFile, outputOptions, isExternal, createPlugins(inputFile, outputFile, sourcemap, production && !min, false, false, null, project), onwarn, useChokidar));

    if (min) {
      const minOutputFile = renamePre(outputFile, MIN_PREFIX);
      configs.push(createConfig(inputFile, { ...outputOptions,
        file: minOutputFile
      }, isExternal, createPlugins(inputFile, minOutputFile, sourcemap, true, false, false, null, project), onwarn, useChokidar));
    }
  }

  if (browserOutput) {
    const {
      input,
      output,
      sourcemap,
      esModule,
      interop,
      format,
      name,
      extend,
      id,
      globals,
      min,
      project
    } = browserOutput;
    const inputFile = findInput(input);

    if (!inputFile) {
      throw inputNotFound('Browser build');
    }

    const outputFile = path.resolve(cwd, output);
    const outputOptions = {
      file: outputFile,
      format,
      sourcemap,
      esModule,
      interop,
      extend,
      globals: globals || {}
    };

    if (format === 'iife' || format === 'umd') {
      if (!name) {
        throw error('option \'name\' is required for IIFE and UMD builds');
      }

      outputOptions.name = name;
    }

    if (id && (format === 'amd' || format === 'umd')) {
      outputOptions.amd = {
        id
      };
    }

    const isBrowserExternal = createIsExternal(globals);
    configs.push(createConfig(inputFile, outputOptions, isBrowserExternal, createPlugins(inputFile, outputFile, sourcemap, production && !min, true, false, null, project), onwarn, useChokidar));

    if (min) {
      const minOutputFile = renamePre(outputFile, MIN_PREFIX);
      configs.push(createConfig(inputFile, { ...outputOptions,
        file: minOutputFile
      }, isBrowserExternal, createPlugins(inputFile, minOutputFile, sourcemap, true, true, false, null, project), onwarn, useChokidar));
    }
  }

  if (binaryOutput) {
    const {
      input,
      output,
      sourcemap,
      esModule,
      interop,
      min,
      project
    } = binaryOutput;
    const inputFile = findInput(input);

    if (!inputFile) {
      throw inputNotFound('Binary build');
    }

    const outputFile = path.resolve(cwd, output);
    const outputOptions = {
      file: outputFile,
      format: 'cjs',
      sourcemap,
      esModule,
      interop,
      exports: 'auto'
    };
    const apiInputFile = commonjsOutput ? commonjsOutput.input : null;
    configs.push(createConfig(inputFile, outputOptions, isExternal, createPlugins(inputFile, outputFile, sourcemap, production && !min, false, true, apiInputFile, project), onwarn, useChokidar));

    if (min) {
      const minOutputFile = renamePre(outputFile, MIN_PREFIX);
      configs.push(createConfig(inputFile, { ...outputOptions,
        file: minOutputFile
      }, isExternal, createPlugins(inputFile, minOutputFile, sourcemap, true, false, true, apiInputFile, project), onwarn, useChokidar));
    }
  }

  return configs;
}

async function configsFromPkg(cwd, options, pkgJson) {
  return pkgToConfigs(await analyzePkg(cwd, pkgJson), options || {});
}

exports.analizePkg = analyzePkg;
exports.analyzePkg = analyzePkg;
exports.configsFromPkg = configsFromPkg;
//# sourceMappingURL=bundlib.js.map
